@@ -1,6 +1,6 @@
 # GNU Solfege - free ear training software
 # vim: set fileencoding=utf-8 :
-# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 Tom Cato Amundsen
+# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2011 Tom Cato Amundsen
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -27,7 +27,6 @@ import stat
 import subprocess
 import sys
 import textwrap
-import time
 
 import gobject
 
@@ -42,9 +41,8 @@ from solfege import uuid
 from solfege import xrandom
 from solfege.dataparser import istr
 from solfege.mpd import elems
-from solfege.mpd import mpdutils
-from solfege.mpd.duration import Duration
-from solfege.mpd.rat import Rat
+from solfege.mpd import Duration
+from solfege.mpd import Rat
 
 _test_mode = False
 
@@ -200,6 +198,10 @@ keywords = (
     'd8', 'p8', 'a8',
     'd9', 'm9', 'M9', 'a9',
     'd10', 'm10', 'M10', 'a10',
+    # clefs:
+    'treble', 'violin', 'french',
+    'subbass', 'bass', 'varbaritone',
+    'baritone', 'tenor', 'alto', 'mezzosoprano', 'soprano',
 )
 predef = {
     'tempo': (60, 4),
@@ -272,13 +274,6 @@ lessonfile_functions = {
         'chordname': (False, lambda s: LabelObject("chordname", s)),
 }
 
-# We need this modified dict to use when parsing data into the
-# sqlite-database. This because we want those data to be in C-locale
-# so that the program can be run in different locales from the same
-# database.
-lessonfile_functions_no_i18n = dict(lessonfile_functions)
-lessonfile_functions_no_i18n['_'] = (False, lambda f: f)
-lessonfile_functions_no_i18n['_i'] = (False, lambda f: f)
 
 def rnc_markup_tokenizer(s):
     """
@@ -710,7 +705,7 @@ class Rvoice(VoiceCommon):
         except mpd.InvalidNotenameException, e:
             errpitch = self.m_musicdata.split("\n")[e.m_lineno][e.m_linepos1:e.m_linepos2]
             if lessonfile_ref.header.random_transpose[0]:
-                return "\\staff\\transpose %s\\relative %s{ %s }" % (
+                return "\\staff\\transpose %s\\relative %s{\n%s\n}" % (
                     lessonfile_ref.m_transpose.get_octave_notename(),
                     errpitch,#self.get_first_pitch().get_octave_notename(),
                     lex.m_string
@@ -722,7 +717,7 @@ class Rvoice(VoiceCommon):
         pitch.m_octave_i = 0
         lex.set_first_pitch(pitch)
         if lessonfile_ref.header.random_transpose[0]:
-            return "\\staff\\transpose %s\\relative %s{ %s }" % (
+            return "\\staff\\transpose %s\\relative %s{\n%s\n}" % (
                 lessonfile_ref.m_transpose.get_octave_notename(),
                 self.get_first_pitch().get_octave_notename(),
                 lex.m_string
@@ -746,6 +741,29 @@ class Rvoice(VoiceCommon):
             self.get_first_pitch().get_octave_notename(),
             self.m_musicdata,
             )
+    def get_err_context(self, exception, lessonfile_ref):
+        """
+        This functions expect exception.m_lineno, .m_linepos1 and .m_linepos2
+        to be set relative to the m_musicdata of the music object,
+        and not relative to the complete mpd string with added \staff{ } etc.
+
+        Return a twoline string showing what caused the exception.
+
+        Rvoice has its own implementation of get_err_context because we
+        will modify the first tone in the music, removing the octave
+        change characters (' and ,) and thus changing the lenght
+        of the string. So we need to adjust the placement of the
+        error marker (^^^^^).
+        """
+        try:
+            delta = abs(self.get_first_pitch().m_octave_i)
+        except mpd.InvalidNotenameException, e:
+            delta = 0
+        if getattr(exception, 'm_obj_lineno', False) is not False:
+            return "\n".join((self.m_musicdata.split("\n")[exception.m_obj_lineno],
+            " " * (exception.m_linepos1 + delta) + "^" * (exception.m_linepos2 - exception.m_linepos1)))
+        return self._mpd_err_context(exception, lessonfile_ref)
+
 
 class Satb(ChordCommon):
     def __init__(self, musicdata):
@@ -978,19 +996,21 @@ class Music(MpdTransposable):
     def get_err_context(self, exception, lessonfile_ref):
         """
         Return a twoline string showing what caused the exception.
+        This method will report wrong error location if there are
+        more than one \staff command in a line.
         """
         first = exception.m_linepos1
         last = exception.m_linepos2
+        tr = r'\transpose %s' % lessonfile_ref.m_transpose.get_octave_notename()
+        delta = len(tr)
+        lines = self.m_musicdata.split("\n")
         if lessonfile_ref.header.random_transpose[0]:
-            s = self.m_musicdata.replace(r'\staff',
-               r'\staff\transpose %s' % lessonfile_ref.m_transpose.get_octave_notename())
-            s = s.replace(r'\addvoice',
-               r'\addvoice\transpose %s' % lessonfile_ref.m_transpose.get_octave_notename())
-            i = len(s) - len(self.m_musicdata)
-        else:
-            i = 0
-        first = exception.m_linepos1 - i
-        last = exception.m_linepos2 - i
+            s = self.m_musicdata.replace(r'\staff', r'\staff%s' % tr)
+            s = s.replace(r'\addvoice', r'\addvoice%s' % tr)
+            if (r'\staff' in lines[exception.m_lineno]
+                or r'\addvoice' in lines[exception.m_lineno]):
+                first = exception.m_linepos1 - delta
+                last = exception.m_linepos2 - delta
         return "\n".join((self.m_musicdata.split("\n")[exception.m_lineno],
             " " * first + "^" * (last - first)))
     def complete_to_musicdata_coords(self, lessonfile_ref, exception):
@@ -1400,7 +1420,10 @@ class TestSupport(object):
         pass the test. (values 0.0 to 1.0)
         """
         m = re.match("([\d\.]+)%", self.header.test_requirement)
-        return float(m.groups()[0])/100.0
+        if m:
+            return float(m.groups()[0])/100.0
+        else:
+            return 0.9
     def enter_test_mode(self):
         self.m_test_questions = self._generate_test_questions()
         self.m_test_idx = -1
@@ -1800,8 +1823,10 @@ class InfoCache(object):
                 'replaces': p.header.get('replaces', []),
         }
         except KeyError:
-            logging.debug("InfoCache.parse_file: FileNotLessonfile(%s)" % filename)
+            logging.debug("InfoCache.parse_file: FileNotLessonfile(%s)", filename)
             raise self.FileNotLessonfile(filename)
+        if not self._data[filename]['title']:
+            self._data[filename]['title'] = "error: empty string as title in '%s'" % filename
     def iter_parse_all_files(self):
         """
         Parse the files and put then in the cache.
@@ -1836,7 +1861,7 @@ class InfoCache(object):
         Parse all standard lesson files and the user_lessonfiles.
         Will not check if reparse is necessary.
         """
-        logging.debug("parse_all_files(when_idle=%s)" % when_idle)
+        logging.debug("parse_all_files(when_idle=%s)", when_idle)
         if when_idle:
             self._lessonfiles_iterator = self.iter_parse_all_files()
             def on_idle_parse():
@@ -1863,7 +1888,7 @@ class InfoCache(object):
             return
         mtime = os.path.getmtime(dir)
         assert isinstance(dir, unicode)
-        logging.debug("cond_parse_dir(%s) mtime=%s" % (dir, mtime))
+        logging.debug("cond_parse_dir(%s) mtime=%s", dir, mtime)
         if dir not in self._dir_mtime:
             for filename in os.listdir(dir):
                 fn = os.path.join(dir, filename)
@@ -1873,10 +1898,10 @@ class InfoCache(object):
                 file_mtime = os.path.getmtime(uri_expand(fn))
                 if fn not in self._data or file_mtime > self._data[fn]['mtime']:
                     try:
-                        logging.debug(" will parse %s. in data:%s" % (fn, fn in self._data))
+                        logging.debug(" will parse %s. in data:%s", fn, fn in self._data)
                         self.parse_file(fn, file_mtime)
                     except self.InfoCacheException:
-                        logging.debug(" exception, not parsed: %s" % fn)
+                        logging.debug(" exception, not parsed: %s", fn)
                         pass
     def iter_user_files(self):
         """
@@ -2002,7 +2027,7 @@ def parse_lesson_file_header(filename):
     # and must be updated there too if changed.
     s = open(filename, 'rU').read(40960)
     m = r.search(s)
-    p = dataparser.Dataparser(predef, lessonfile_functions_no_i18n)
+    p = dataparser.Dataparser(predef, lessonfile_functions)
     p.m_location = os.path.split(filename)
     p.m_ignore_lookup_error = True
     if not m:
